<?php

/**
-------------------------------------------------------------------------
mediamallfactory - Media Mall Factory 4.2.2
-------------------------------------------------------------------------
 * @author thePHPfactory
 * @copyright Copyright (C) 2011 SKEPSIS Consult SRL. All Rights Reserved.
 * @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Websites: http://www.thePHPfactory.com
 * Technical Support: Forum - http://www.thePHPfactory.com/forum/
-------------------------------------------------------------------------
*/

defined('_JEXEC') or die;

class FactoryForm extends JForm
{
    public function loadFile($file, $reset = true, $xpath = false)
    {
        if (!parent::loadFile($file, $reset, $xpath)) {
            return false;
        }

        $this->setLabels();

        return true;
    }

    public static function getInstance($name, $data = null, $options = array(), $replace = true, $xpath = false)
    {
        self::addPaths($options);

        // Reference to array with form instances
        $forms = &self::$forms;

        // Only instantiate the form if it does not already exist.
        if (!isset($forms[$name])) {
            $data = trim($data);

            if (empty($data)) {
                throw new Exception(JText::_('JLIB_FORM_ERROR_NO_DATA'));
            }

            // Instantiate the form.
            $forms[$name] = new self($name, $options);

            // Load the data.
            if (substr(trim($data), 0, 1) == '<') {
                if ($forms[$name]->load($data, $replace, $xpath) == false) {
                    throw new Exception(JText::_('JLIB_FORM_ERROR_XML_FILE_DID_NOT_LOAD'));

                    return false;
                }
            } else {
                if ($forms[$name]->loadFile($data, $replace, $xpath) == false) {
                    throw new Exception(JText::_('JLIB_FORM_ERROR_XML_FILE_DID_NOT_LOAD'));

                    return false;
                }
            }
        }

        return $forms[$name];
    }

    public function render($tmpl = 'template', $path = null)
    {
        jimport('joomla.filesystem.file');

        if (is_null($path)) {
            $path = FactoryApplication::getInstance()->getPath('factory') . DS . 'form';
        }

        $template = $path . DS . $tmpl . '.php';
        if (!JFile::exists($template)) {
            throw new Exception(FactoryText::_('form_template_file_not_found'), 500);
        }

        ob_start();
        include $template;
        $output = ob_get_contents();
        ob_end_clean();

        return $output;
    }

    public function renderFieldset($name, $tmpl = 'fieldset', $path = null)
    {
        jimport('joomla.filesystem.file');

        if (is_null($path)) {
            $path = FactoryApplication::getInstance()->getPath('factory') . DS . 'form';
        }

        $template = $path . DS . $tmpl . '.php';
        if (!JFile::exists($template)) {
            throw new Exception(FactoryText::_('form_template_file_not_found'), 500);
        }

        $this->fieldset = $name;

        ob_start();
        include $template;
        $output = ob_get_contents();
        ob_end_clean();

        return $output;
    }

    protected function renderHiddenFields()
    {
        $html = array();

        $elements = $this->xml->xpath('//fieldset/field[@type="hidden"]');

        foreach ($elements as $element) {
            if ($field = $this->loadField($element)) {
                $html[] = $field->input;
            }
        }

        $html[] = JHtml::_('form.token');

        return implode("\n", $html);
    }

    protected function setLabels()
    {
        $admin = JFactory::getApplication()->isAdmin();

        $name = $this->name;
        if (false !== strpos($name, '.')) {
            list($option, $name) = explode('.', $name);
        }

        foreach ($this->getFieldsets() as $fieldset) {
            // Set the fieldset label.
            if ('' == $fieldset->label) {
                $tmp = $this->xml->xpath('/form/fieldset[@name="' . $fieldset->name . '"]');
                $tmp[0]['label'] = FactoryText::_('form_' . $name . '_fieldset_' . $fieldset->name . '_label');
            }

            foreach ($this->getFieldset($fieldset->name) as $field) {
                // Check if field is only for backend.
                if (!$admin && 'true' === $this->getFieldAttribute($field->fieldname, 'adminonly', '', $field->group)) {
                    $this->removeField($field->fieldname, $field->group);
                    continue;
                }

                // Set field label.
                if ('' == $this->getFieldAttribute($field->fieldname, 'label', '', $field->group)) {
                    $this->setFieldAttribute(
                        $field->fieldname,
                        'label',
                        FactoryText::_('form_' . $name . '_' . $field->fieldname . '_label'),
                        $field->group);
                }

                // Set field description.
                if ('' == $this->getFieldAttribute($field->fieldname, 'description', '', $field->group)) {
                    $this->setFieldAttribute(
                        $field->fieldname,
                        'description',
                        FactoryText::_('form_' . $name . '_' . $field->fieldname . '_desc'),
                        $field->group);
                }
            }
        }

        return true;
    }

    protected static function addPaths($options)
    {
        $location = isset($options['path_location']) ? '_' . $options['path_location'] : '';
        $base = FactoryApplication::getInstance()->getPath('component' . $location) . DS . 'models';

        self::addFormPath($base . DS . 'forms');
        self::addFieldPath($base . DS . 'fields');
        self::addRulePath($base . DS . 'rules');

        return true;
    }

    public static function addFieldPath($location = 'site')
    {
        if (in_array($location, array('site', 'admin'))) {
            $location = FactoryApplication::getInstance()->getPath('component_' . $location) . DS . 'models' . DS . 'fields';
        }

        return parent::addFieldPath($location);
    }

    protected function getConfigTabs()
    {
        $sets = array();
        $fieldsets = $this->xml->xpath('//fieldset');

        foreach ($fieldsets as $fieldset) {
            $fields = $this->xml->xpath('//fieldset[@name="' . (string)$fieldset->attributes()->name . '"] //fields //fields');

            foreach ($fields as $field) {
                $sets[(string)$fieldset->attributes()->name][(string)$field->attributes()->side][] = (string)$field->attributes()->name;
            }
        }

        return $sets;
    }

    protected function getConfigFields($fieldset, $name)
    {
        $array = array();
        $fieldsets = $this->xml->xpath('//fieldset[@name="' . $fieldset . '"] //fields //fields[@name="' . $name . '"]');

        if (!$fieldsets) {
            return $array;
        }

        $temp = $fieldsets[0]->xpath('ancestor::fields/@name');
        $set = (string)$temp[0];

        foreach ($fieldsets[0]->field as $element) {
            $group = $set . '.' . $name;

            if ($field = $this->loadField($element, $group)) {
                $array[$field->id] = $field;
            }
        }

        return $array;
    }

    public function removeFieldset($fieldset)
    {
        // Make sure there is a valid JForm XML document.
        if (!($this->xml instanceof SimpleXMLElement)) {
            return false;
        }

        // Get the fields elements for a given group.
        $elements = $this->xml->xpath('//fieldset[@name="' . $fieldset . '"]');
        foreach ($elements as &$element) {
            $dom = dom_import_simplexml($element);
            $dom->parentNode->removeChild($dom);
        }

        return true;
    }

    protected function &findFieldsByFieldset($name)
    {
        $false = false;

        // Make sure there is a valid JForm XML document.
        if (!($this->xml instanceof SimpleXMLElement)) {
            return $false;
        }

        $fields = $this->xml->xpath('//fieldset[@name="' . $name . '"]//field | //field[@fieldset="' . $name . '"]');

        return $fields;
    }
}
