<?php

/**
-------------------------------------------------------------------------
mediamallfactory - Media Mall Factory 4.2.2
-------------------------------------------------------------------------
 * @author thePHPfactory
 * @copyright Copyright (C) 2011 SKEPSIS Consult SRL. All Rights Reserved.
 * @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Websites: http://www.thePHPfactory.com
 * Technical Support: Forum - http://www.thePHPfactory.com/forum/
-------------------------------------------------------------------------
*/

defined('_JEXEC') or die;

class FactoryTable extends JTable
{
    protected $_errors = array();

    public static function getInstance($type, $prefix = 'JTable', $config = array())
    {
        $prefix = FactoryApplication::getInstance()->getComponent() . 'Table';
        JTable::addIncludePath(FactoryApplication::getInstance()->getPath('component_administrator') . DS . 'tables');

        return parent::getInstance($type, $prefix, $config);
    }

    public function setError($error)
    {
        array_push($this->_errors, $error);
    }

    public function getErrors()
    {
        return $this->_errors;
    }

    public function getError($i = null, $toString = true)
    {
        // Find the error
        if ($i === null) {
            // Default, return the last message
            $error = end($this->_errors);
        } elseif (!array_key_exists($i, $this->_errors)) {
            // If $i has been specified but does not exist, return false
            return false;
        } else {
            $error = $this->_errors[$i];
        }

        // Check if only the string is requested
        if ($error instanceof Exception && $toString) {
            return (string)$error;
        }

        return $error;
    }

    public function store($updateNulls = false)
    {
        $key = $this->_tbl_key;

        if (property_exists($this, 'created_at') && !$this->$key) {
            $this->created_at = JFactory::getDate()->toSql();
        }

        if (property_exists($this, 'updated_at')) {
            $this->updated_at = JFactory::getDate()->toSql();
        }

        return parent::store($updateNulls);
    }

    public function load($keys = null, $reset = true)
    {
        if (!parent::load($keys, $reset)) {
            return false;
        }

        // Convert params.
        if (property_exists($this, 'params')) {
            $this->params = new JRegistry($this->params);
        }

        return true;
    }

    public function bind($src, $ignore = array())
    {
        if (is_array($src) && isset($src['params']) && is_array($src['params'])) {
            $registry = new JRegistry($src['params']);
            $src['params'] = $registry->toString();
        }

        return parent::bind($src, $ignore);
    }

    public function delete($pk = null)
    {
        $k = $this->_tbl_key;
        $pk = (is_null($pk)) ? $this->$k : $pk;

        if (!parent::delete($pk)) {
            return false;
        }

        $event = $this->getName() . 'Delete';

        FactoryApplication::getInstance()->trigger($event, array($pk));

        return true;
    }

    public function publish($pks = null, $state = 1, $userId = 0)
    {
        $state = is_null($pks) ? 1 : $pks;
        $k = $this->_tbl_key;

        $dbo = $this->getDbo();
        $query = $dbo->getQuery(true)
            ->update($this->_tbl)
            ->set('published = ' . (int)$state);
        $query->where($k . ' = ' . $dbo->quote($this->$k));

        $dbo->setQuery($query);

        try {
            $dbo->execute();
        } catch (Exception $e) {
            $this->setError($e->getMessage());
            return false;
        }

        $event = $this->getName() . 'Publish';

        FactoryApplication::getInstance()->trigger($event, array($this->$k, $state, $this->user_id));

        return true;
    }

    protected function getName()
    {
        return str_replace(FactoryApplication::getInstance()->getComponent() . 'Table', '', get_class($this));
    }
}
