<?php

/**
-------------------------------------------------------------------------
mediamallfactory - Media Mall Factory 4.2.2
-------------------------------------------------------------------------
 * @author thePHPfactory
 * @copyright Copyright (C) 2011 SKEPSIS Consult SRL. All Rights Reserved.
 * @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Websites: http://www.thePHPfactory.com
 * Technical Support: Forum - http://www.thePHPfactory.com/forum/
-------------------------------------------------------------------------
*/

defined('_JEXEC') or die;

class JHtmlMediaMallFactoryMedia
{
    public static function rating($rating, $votes, $media_id)
    {
        $html = array();

        $html[] = '<div class="rating">';
        $html[] = self::simpleRating($rating);

        $html[] = '<div class="small muted">';

        if ($votes) {
            $url = FactoryRoute::view('media&media_id=' . $media_id . '#reviews');
            $link = '<a href="' . $url . '">' . FactoryText::plural('media_count_reviews', $votes) . '</a>';

            $html[] = FactoryText::sprintf('media_count_reviews_from_x_reviews', $link);
        } else {
            $html[] = FactoryText::plural('media_count_reviews', 0);
        }

        $html[] = '</div>';
        $html[] = '</div>';

        return implode("\n", $html);
    }

    public static function simpleRating($rating = 0)
    {
        FactoryHtml::stylesheet('rating');

        $html = array();
        $used = 0;

        $html[] = '<div class="rating-wrapper">';

        for ($i = 0; $i < floor($rating); $i++) {
            $html[] = '<div class="rating-star star-full"></div>';
            $used++;
        }

        if ($rating - floor($rating) >= .5) {
            $html[] = '<div class="rating-star star-half"></div>';
            $used++;
        }

        for ($i = $used; $i < 5; $i++) {
            $html[] = '<div class="rating-star star-empty"></div>';
        }

        $html[] = '</div>';

        return implode("\n", $html);
    }

    public static function mediaCountLink($id, $count)
    {
        $html = array();

        if ($count) {
            $html[] = '<span class="media_count">';
            $html[] = '(' . JHtml::_('Factory.link', 'list&filter[category]=' . $id, FactoryText::plural('media_count_files', $count)) . ')';
            $html[] = '</span>';
        }

        return implode("\n", $html);
    }

    public static function archiveIcon($archive = false)
    {
        $html = array();

        if ($archive) {
            $html[] = '<div class="archive-icon hasTip" title="' . FactoryText::_('media_archive_icon_title') . '::' . FactoryText::_('media_archive_icon_text') . '"></div>';
        }

        return implode("\n", $html);
    }

    public static function prices($media)
    {
        $html = array();

        $html[] = '<ul class="prices">';

        // Media price.
        if ($media->has_media) {
            $html[] = '<li>';

            if ($media->cost_media) {
                $html[] = '<div class="price">' . FactoryText::plural('media_view_credits_simple', $media->cost_media) . '</div>';
                $html[] = '<span class="muted small">' . FactoryText::plural('media_purchase_media_link', $media->type_views) . '</span>';
            } else {
                $html[] = '<div class="price price-free">' . FactoryText::_('media_prices_free_media') . '</div>';
                $html[] = '<span class="muted small">' . FactoryText::_('media_prices_free_media_info') . '</span>';
            }

            $html[] = '</li>';
        }

        // Archive price.
        if ($media->has_archive) {
            $html[] = '<li>';

            if ($media->cost_archive) {
                $html[] = '<div class="price">' . FactoryText::plural('media_view_credits_simple', $media->cost_archive) . '</div>';
                $html[] = '<span class="muted small">' . FactoryText::_('media_prices_purchase_download_info') . '</span>';
            } else {
                $html[] = '<div class="price price-free">' . FactoryText::_('media_prices_free_archive') . '</div>';
                $html[] = '<span class="muted small">' . FactoryText::_('media_prices_free_archive_info') . '</span>';
            }

            $html[] = '</li>';
        }

        // Contact price.
        $params = new \Joomla\Registry\Registry($media->profile_params);
        if ($params->get('allow_contact', 0)) {
            $contactCost = FactoryApplication::getInstance()->getParam('general.credits.cost_contact', 1);

            $html[] = '<li>';
            $html[] = '<div class="price">' . FactoryText::plural('media_view_credits_simple', $contactCost) . '</div>';
            $html[] = '<span class="muted small">' . FactoryText::_('media_prices_purchase_contact_info') . '</span>';
            $html[] = '</li>';
        }

        $html[] = '</ul>';

        return implode("\n", $html);
    }

    public static function mediaLink($id, $title)
    {
        if ('' == $title) {
            return FactoryText::_('media_removed');
        }

        $html = array();

        $html[] = '<a href="' . FactoryRoute::view('media&media_id=' . $id) . '">';
        $html[] = $title;
        $html[] = '</a>';

        return implode("\n", $html);
    }

    public static function purchaseViewsButton($cost)
    {
        $html = array();

        if ($cost) {
            $html[] = '<div class="purchase-button">';
            $html[] = '<input type="button" value="' . FactoryText::_('media_purchase_media_views') . '" />';
            $html[] = '</div>';
        }

        return implode("\n", $html);
    }

    public static function thumbnail($thumbnail)
    {
        $html = array();

        $html[] = '<div class="media-photo ' . ($thumbnail ? '' : 'no-photo') . '">';

        if ($thumbnail) {
            $html[] = '<img src="' . $thumbnail . '" />';
        }

        $html[] = '</div>';

        return implode('', $html);
    }

    public static function published($state)
    {
        $html = array();

        $classes = array(
            -1 => 'warning',
            0  => 'important',
            1  => 'success',
        );

        $texts = array(
            -1 => FactoryText::_('media_status_pending'),
            0  => FactoryText::_('media_status_unpublished'),
            1  => FactoryText::_('media_status_published'),
        );

        $html[] = '<span class="factory-label label-' . $classes[$state] . '">';
        $html[] = $texts[$state];
        $html[] = '</span>';

        return implode("\n", $html);
    }

    public static function unread($count = null)
    {
        $html = array();

        if ($count) {
            $html[] = '<span class="factory-badge">' . $count . '</span>';
        }

        return implode("\n", $html);
    }

    public static function categoryPurchaseLink($category_id, $cost)
    {
        if (!$cost) {
            return '';
        }

        $html = '<p><a href="' . FactoryRoute::task('media.purchasecategory&category_id=' . $category_id) . '">' . FactoryText::sprintf('category_purchase', $cost) . '</a></p>';

        return $html;
    }
}
