<?php

/**
-------------------------------------------------------------------------
mediamallfactory - Media Mall Factory 4.2.2
-------------------------------------------------------------------------
 * @author thePHPfactory
 * @copyright Copyright (C) 2011 SKEPSIS Consult SRL. All Rights Reserved.
 * @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * Websites: http://www.thePHPfactory.com
 * Technical Support: Forum - http://www.thePHPfactory.com/forum/
-------------------------------------------------------------------------
*/

defined('_JEXEC') or die;

defined('DS') or define('DS', DIRECTORY_SEPARATOR);

class plgSearchMediaMallFactory extends JPlugin
{
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);

        $this->loadLanguage();
    }

    public function onContentSearchAreas()
    {
        static $areas = array(
            'com_mediamallfactory' => 'PLG_MEDIAMALLFACTORY_SEARCH_CONTENT_AREA',
        );

        return $areas;
    }

    public function onContentSearch($text, $phrase = '', $ordering = '', $areas = null)
    {
        // Initialise variables.
        $text = trim($text);
        $limit = $this->params->get('limit', 10);
        $search = $this->params->get('search', array());

        // Check if search is allowed.
        if (!$this->allowSearch($text, $areas, $search)) {
            return array();
        }

        // Get the results.
        $results = $this->getResults($phrase, $text, $search, $ordering, $limit);

        // Parse the results.
        foreach ($results as $result) {
            $result->href = JRoute::_('index.php?option=com_mediamallfactory&view=media&media_id=' . $result->id);
            $result->browsernav = 2;
            $result->text = '' == $result->text ? $result->title : $result->text;
        }

        return $results;
    }

    protected function getResults($phrase, $text, $search, $ordering, $limit)
    {
        $dbo = JFactory::getDbo();

        // Create the query.
        $query = $dbo->getQuery(true)
            ->select('m.id, m.title, m.description AS text')
            ->select('m.created_at AS created, c.title AS section')
            ->from('#__mediamallfactory_media m')
            ->leftJoin('#__categories c ON c.id = m.category_id')
            ->where('m.published = ' . $dbo->quote(1));

        // Filter results by phrase.
        $query->where($this->getWhereCondition($phrase, $text, $dbo, $search));

        // Sort results.
        $query->order($this->getOrderCondition($ordering));

        $results = $dbo->setQuery($query, 0, $limit)
            ->loadObjectList();

        return $results;
    }

    protected function allowSearch($text, $areas, $search)
    {
        // Check if component is installed.
        $extension = JTable::getInstance('Extension');
        $result = $extension->find(array('element' => 'com_mediamallfactory', 'type' => 'component'));
        if (!$result) {
            return false;
        }

        // Check if any fields have been selected for search.
        if (!is_array($search) || !$search) {
            return false;
        }

        // Check if any text was entered.
        if ($text == '') {
            return false;
        }

        // Check if areas is set and if area is allowed.
        if (is_array($areas)) {
            if (!array_intersect($areas, array_keys($this->onContentSearchAreas()))) {
                return false;
            }
        }

        return true;
    }

    protected function getWhereCondition($phrase, $text, $dbo, $search)
    {
        switch ($phrase) {
            case 'exact':
                $operand = ' LIKE ' . $dbo->quote('%' . $text . '%');
                $where = '(' . implode($operand . ' OR ', $search) . $operand . ')';
                break;

            case 'all':
            case 'any':
            default:
                $words = explode(' ', $text);
                $where = array();

                foreach ($words as $word) {
                    $operand = ' LIKE ' . $dbo->quote('%' . $word . '%');
                    $where[] = '(' . implode($operand . ' OR ', $search) . $operand . ')';
                }

                $operand = 'all' == $phrase ? ' AND ' : ' OR ';
                $where = implode($operand, $where);
                break;
        }

        return $where;
    }

    protected function getOrderCondition($ordering)
    {
        switch ($ordering) {
            case 'oldest':
                $order = 'm.created_at ASC';
                break;

            case 'popular':
                $order = 'm.rating DESC';
                break;

            case 'alpha':
                $order = 'm.title ASC';
                break;

            case 'category':
                $order = 'c.title ASC, m.title ASC';
                break;

            case 'newest':
            default:
                $order = 'm.created_at DESC';
                break;
        }

        return $order;
    }
}
